@@ -18,6 +18,8 @@
 #include <restinio/request_handler.hpp>
 #include <restinio/traits.hpp>
 
+#include <restinio/variant.hpp>
+
 namespace restinio
 {
 
@@ -418,6 +420,48 @@ struct ip_blocker_holder_t< ip_blocker::noop_ip_blocker_t >
 	}
 };
 
+//
+// acceptor_post_bind_hook_t
+//
+/*!
+ * @brief A type of callback to be called after a successful invocation
+ * of bind() function for the acceptor.
+ *
+ * @since v.0.6.11
+ */
+using acceptor_post_bind_hook_t = std::function<
+		void(asio_ns::ip::tcp::acceptor &) >;
+
+namespace details
+{
+
+//
+// no_address_specified_t
+//
+/*!
+ * @brief A special indicator for the case when IP address for a server
+ * is not set explicitly.
+ *
+ * @since v.0.6.11
+ */
+struct no_address_specified_t {};
+
+//
+// address_variant_t
+//
+/*!
+ * @brief A type of variant for holding IP address for a server in
+ * various representations.
+ *
+ * @since v.0.6.11
+ */
+using address_variant_t = variant_t<
+		no_address_specified_t,
+		std::string,
+		asio_ns::ip::address >;
+
+} /* namespace details */
+
 //
 // basic_server_settings_t
 //
@@ -475,6 +519,7 @@ class basic_server_settings_t
 			return std::move( this->port( p ) );
 		}
 
+		RESTINIO_NODISCARD
 		std::uint16_t
 		port() const
 		{
@@ -494,12 +539,30 @@ class basic_server_settings_t
 			return std::move( this->protocol( p ) );
 		}
 
+		RESTINIO_NODISCARD
 		asio_ns::ip::tcp
 		protocol() const
 		{
 			return m_protocol;
 		}
 
+		/*!
+		 * Sets the IP address for a server in textual form.
+		 *
+		 * Usage example:
+		 * @code
+		 * using my_server_t = restinio::http_server_t< my_server_traits_t >;
+		 * my_server_t server{
+		 * 	restinio::own_io_context(),
+		 * 	[](auto & settings) {
+		 * 		settings.port(8080);
+		 * 		settings.address("192.168.1.1");
+		 * 		settings.request_handler(...);
+		 * 		...
+		 * 	}
+		 * };
+		 * @endcode
+		 */
 		Derived &
 		address( std::string addr ) &
 		{
@@ -507,13 +570,74 @@ class basic_server_settings_t
 			return reference_to_derived();
 		}
 
+		/*!
+		 * Sets the IP address for a server in textual form.
+		 *
+		 * Usage example:
+		 * @code
+		 * restinio::run(
+		 * 	restinio::on_this_thread()
+		 * 		.port(...)
+		 * 		.address("192.168.1.1")
+		 * 		.request_handler(...)
+		 * 	);
+		 * @endcode
+		 */
 		Derived &&
 		address( std::string addr ) &&
 		{
 			return std::move( this->address( std::move( addr ) ) );
 		}
 
-		const std::string &
+		/*!
+		 * Sets the IP address for a server in binary form.
+		 *
+		 * Usage example:
+		 * @code
+		 * auto actual_ip = asio::ip::address::from_string(app.config().ip_addr());
+		 * ...
+		 * using my_server_t = restinio::http_server_t< my_server_traits_t >;
+		 * my_server_t server{
+		 * 	restinio::own_io_context(),
+		 * 	[actual_ip](auto & settings) {
+		 * 		settings.port(8080);
+		 * 		settings.address(actual_ip);
+		 * 		settings.request_handler(...);
+		 * 		...
+		 * 	}
+		 * };
+		 * @endcode
+		 */
+		Derived &
+		address( asio_ns::ip::address addr ) &
+		{
+			m_address = addr;
+			return reference_to_derived();
+		}
+
+		/*!
+		 * Sets the IP address for a server in binary form.
+		 *
+		 * Usage example:
+		 * @code
+		 * auto actual_ip = asio::ip::address::from_string(app.config().ip_addr());
+		 * ...
+		 * restinio::run(
+		 * 	restinio::on_this_thread()
+		 * 		.port(...)
+		 * 		.address(actual_ip)
+		 * 		.request_handler(...)
+		 * 	);
+		 * @endcode
+		 */
+		Derived &&
+		address( asio_ns::ip::address addr ) &&
+		{
+			return std::move( this->address( addr ) );
+		}
+
+		RESTINIO_NODISCARD
+		const details::address_variant_t &
 		address() const
 		{
 			return m_address;
@@ -872,6 +996,13 @@ class basic_server_settings_t
 			return std::move(this->cleanup_func( std::forward<Func>(func) ));
 		}
 
+		/*!
+		 * @note
+		 * This method is intended to be used by RESTinio and it can be
+		 * changed or removed in future versions of RESTinio without any
+		 * notice.
+		 */
+		RESTINIO_NODISCARD
 		cleanup_functor_t
 		giveaway_cleanup_func()
 		{
@@ -1111,6 +1242,82 @@ class basic_server_settings_t
 			this->check_valid_ip_blocker_pointer();
 		}
 
+		// Acceptor post-bind hook.
+		/*!
+		 * @brief A setter for post-bind callback.
+		 *
+		 * Usage example:
+		 * @code
+		 * using my_server_t = restinio::http_server_t< my_server_traits_t >;
+		 * my_server_t server{
+		 * 	restinio::own_io_context(),
+		 * 	[](auto & settings) {
+		 * 		settings.port(...);
+		 * 		settings.address(...);
+		 * 		settings.acceptor_post_bind_hook(
+		 * 			[](asio::ip::tcp::acceptor & acceptor) {
+		 * 				...
+		 * 			})
+		 * 		settings.request_handler(...);
+		 * 		...
+		 * 	}
+		 * };
+		 * @endcode
+		 *
+		 * @since v.0.6.11
+		 */
+		Derived &
+		acceptor_post_bind_hook( acceptor_post_bind_hook_t hook ) &
+		{
+			if( !hook )
+				throw exception_t{ "acceptor_post_bind_hook cannot be empty" };
+
+			m_acceptor_post_bind_hook = std::move(hook);
+			return reference_to_derived();
+		}
+
+		/*!
+		 * @brief A setter for post-bind callback.
+		 *
+		 * Usage example:
+		 * @code
+		 * restinio::run(
+		 * 	restinio::on_this_thread()
+		 * 		.port(...)
+		 * 		.address(...)
+		 * 		.acceptor_post_bind_hook(
+		 * 			[](asio::ip::tcp::acceptor & acceptor) {
+		 * 				...
+		 * 			})
+		 * 		.request_handler(...)
+		 * 	);
+		 * @endcode
+		 *
+		 * @since v.0.6.11
+		 */
+		Derived &&
+		acceptor_post_bind_hook( acceptor_post_bind_hook_t hook ) &&
+		{
+			return std::move(this->acceptor_post_bind_hook( std::move(hook) ));
+		}
+
+		/*!
+		 * @brief A getter for post-bind callback.
+		 *
+		 * @note
+		 * This method is intended to be used by RESTinio and it can be
+		 * changed or removed in future versions of RESTinio without any
+		 * notice.
+		 *
+		 * @since v.0.6.11
+		 */
+		RESTINIO_NODISCARD
+		acceptor_post_bind_hook_t
+		giveaway_acceptor_post_bind_hook()
+		{
+			return std::move(m_acceptor_post_bind_hook);
+		}
+
 	private:
 		Derived &
 		reference_to_derived()
@@ -1144,7 +1351,11 @@ class basic_server_settings_t
 		//! \{
 		std::uint16_t m_port;
 		asio_ns::ip::tcp m_protocol;
-		std::string m_address;
+		/*!
+		 * @note
+		 * This member has type address_variant_t since v.0.6.11
+		 */
+		details::address_variant_t m_address;
 		//! \}
 
 		//! Size of buffer for io operations.
@@ -1177,6 +1388,16 @@ class basic_server_settings_t
 		//! Acceptor options setter.
 		std::unique_ptr< acceptor_options_setter_t > m_acceptor_options_setter;
 
+		//! A hook to be called just after a successful call to bind for acceptor.
+		/*!
+		 * An empty lambda is used by default.
+		 *
+		 * @since v.0.6.11
+		 */
+		acceptor_post_bind_hook_t m_acceptor_post_bind_hook{
+				[](asio_ns::ip::tcp::acceptor &) {}
+			};
+
 		//! Socket options setter.
 		std::unique_ptr< socket_options_setter_t > m_socket_options_setter;
 
